@@ -6,6 +6,8 @@ from textwrap import dedent
 from bs4 import BeautifulSoup as bs
 from argparse import ArgumentParser, SUPPRESS
 from pkg_resources import resource_filename
+from .helpers import better_pdataframe_from_data
+from .config import get_config
 
 import pystata
 import sfi
@@ -27,32 +29,19 @@ def count():
 class SelVar():
     """
     Class for generating selection var in Stata
-    TO DO: Keep track of names being used
     """
     def __init__(self,condition):
-        condition = condition.replace('if ','').strip()
+        condition = condition.replace('if ','',1).strip()
         if condition == '':
             self.varname = None
         else:
-            self.varname = "pystataTmp_" + str(random.randrange(1000,9999))      
-            cmd = f"gen {self.varname} = cond({condition},1,0)"
-            pystata.stata.run(cmd, quietly=True)        
-        
+            cmd = f"tempvar __selectionVar\ngenerate `__selectionVar' = cond({condition},1,0)"
+            pystata.stata.run(cmd, quietly=True)      
+            self.varname = sfi.Macro.getLocal("__selectionVar")  
+
     def clear(self):
         if self.varname != None:
-            pystata.stata.run(f"capture drop {self.varname}", quietly=True)            
-
-
-def genSelVar(condition):
-    condition = condition.replace(' if ','').strip()
-    # TODO: create a tmp var class to keep track of things
-    varname = "pystataTmp_" + str(random.randrange(1000,9999))
-    
-    # Gen selection var in Stata
-    cmd = f"gen {varname} = cond({condition},1,0)"
-    pystata.stata.run(cmd, quietly=True)
-
-    return varname
+            pystata.stata.run(f"capture drop {self.varname}", quietly=True)     
 
 def InVar(code):
     """
@@ -73,13 +62,13 @@ class StataMagics():
     html_help = urllib.parse.urljoin(html_base, "help.cgi?{}")
 
     magic_regex = re.compile(
-        #r'\A(%|\*%)(?P<magic>.+?)(?P<code>\s+.*)?\Z', flags=re.DOTALL + re.MULTILINE)
-        r'\A(%|\*%)(?P<magic>.+?)(?P<code>\s+(?!if\s)(?!in\s).+?)?(?P<if>\sif\s+.+?)?(?P<in>\s+in\s+.+?)?\Z', flags=re.DOTALL + re.MULTILINE)
+        r'\A(%|\*%)(?P<magic>.+?)(?P<code>\s+(?!if\s)(?!\sif)(?!in\s)(?!\sin).+?)?(?P<if>\s+if\s+.+?)?(?P<in>\s+in\s+.+?)?\Z', flags=re.DOTALL + re.MULTILINE)
 
     # Format: magic_name: help_content
     available_magics = {
-        'browse': '{} [-h] [N] [varlist] [if]',
-        'help': '{} [-h] command_or_topic_name'
+        'browse': '{} [-h] [N] [varlist] [if] [in]',
+        'help': '{} [-h] command_or_topic_name',
+        'quietly': ''
     }
     
     csshelp_default = resource_filename(
@@ -107,9 +96,12 @@ class StataMagics():
         return code        
 
     def magic_browse(self,args,kernel):
+        """
+        Display data in a nicely-formatted table.
+        """
+        env = get_config()
         N_max = 200
-        vars = None
-
+        
         # If and in statements
         sel_var = SelVar(args['if'])
         start,end = InVar(args['in'])
@@ -123,8 +115,7 @@ class StataMagics():
                 del vargs[0]    
 
         # Specified variables?
-        if len(vargs) >= 1:
-            vars = vargs
+        vars = vargs if len(vargs) >= 1 else None
 
         # Obs range
         if start != None and end != None:
@@ -132,14 +123,20 @@ class StataMagics():
         else:
             obs_range = range(0,min(count(),N_max))
 
+        # Missing value display format
+        missingval = np.NaN
+        if env['missing'] is not None and env['missing'] != 'pandas':
+            missingval = env['missing']  
+        
         try:
-            df = pystata.stata.pdataframe_from_data(obs=obs_range,
+            df = better_pdataframe_from_data(obs=obs_range,
                                                     var=vars,
                                                     selectvar=sel_var.varname,
-                                                    missingval='')
+                                                    missingval=missingval)
             if vars == None and sel_var.varname != None:
                 df = df.drop([sel_var.varname],axis=1)
-            html = df.to_html(na_rep='.', notebook=True)
+                
+            html = df.to_html(notebook=True)
 
             content = {
                     'data': {
@@ -153,12 +150,14 @@ class StataMagics():
         if sel_var != None:
             # Drop selection var in Stata. We put this outside of try to ensure 
             # the temp variable gets deleted even when there is an error.
-            #pystata.stata.run(f"capture drop {sel_var}", quietly=True)            
             sel_var.clear()
 
         return ''
 
     def magic_help(self,args,kernel):
+        """
+        Show help file from stata.com.
+        """
 
         code = args['code']
 
